%include "./asm_io.inc"    ; Include I/O library for all I/O operations

; --- Data Segment ---

segment .data
    welcome_msg db "Hello Thanks for playing - may the ods ever be in your favour, ", 0 ; Welcome message string
    prompt_name db "Please enter your Name (mine is Bob): ", 0 ; Prompt message for entering name
    prompt_count db "Enter number of times to print (between 50 and 100): ", 0 ; Prompt for number of repetitions
    prompt_start db "Enter the start of the range (1 to 100): ", 0 ; Prompt for range start
    prompt_end db "Enter the end of the range (1 to 100): ", 0 ; Prompt for range end
    range_error_msg db "Error: Invalid range.", 0 ; Error message for invalid range
    sum_result_msg db "The sum of numbers in the range is: ", 0 ; Result message for range sum
    error_msg db "Error: Number must be between 50 and 100.", 0 ; Error message for invalid count
    newline db 10, 0 ; Newline character

; --- BSS Segment ---

segment .bss
    name resb 30 ; Reserve 30 bytes for storing the user's name
    count resd 1 ; Reserve space for storing the count
    array resd 100 ; Reserve space for 100 integers
    sum resd 1 ; Reserve space for the calculated sum
    start_range resd 1 ; Reserve space for the start of the range
    end_range resd 1 ; Reserve space for the end of the range

; --- Code Segment ---

segment .text
    global asm_main ; Make asm_main accessible externally

asm_main:
    pusha ; Save the current state of all general-purpose registers

    ; --- Prompt for entering and reading the user’s name ---
    
    mov eax, prompt_name ; Load prompt_name address into EAX
    call print_string ; Display the name prompt
    mov edi, name ; Move address to store the name into EDI
    mov ecx, 30 ; Set loop counter to 30 for name input length
    xor al, al ; Clear AL for storing characters

read_name_loop:
    call read_char ; Read a character from the user
    cmp al, 10 ; Compare with newline character
    je name_input_complete ; If newline, finish input
    stosb ; Store the character in AL to the memory at EDI
    loop read_name_loop ; Repeat for all characters

name_input_complete:
    mov byte [edi], 0 ; Null-terminate the user's inputted name

    ; --- Prompt for entering and reading the count ---
    
    mov eax, prompt_count ; Load address of prompt_count into EAX
    call print_string ; Display the count prompt
    call read_int ; Read the integer input
    mov [count], eax ; Store the user's inputted count

    ; --- Validating count ---
    
    mov eax, [count] ; Move count into EAX for validation
    cmp eax, 50 ; Compare count with 50
    jl display_error ; Jump to error if less than 50
    cmp eax, 100 ; Compare count with 100
    jg display_error ; Jump to error if greater than 100

    ; --- Loop for printing the welcome message ---
    
    mov ecx, eax ; Load count into ECX as loop counter

print_welcome:
    mov eax, welcome_msg ; Load address of welcome_msg into EAX
    call print_string ; Print the welcome message
    mov eax, name ; Load address of name into EAX
    call print_string ; Print the user's name
    call print_nl ; Print a newline after each message
    loop print_welcome ; Repeat for count times

    ; --- Initialise my array with values from 1 to 100 ---

    mov ecx, 100 ; Set loop counter `ECX` to 100
    mov eax, 1 ; Start with the number 1
    mov ebx, array ; Point `EBX` to the start of the array (index 0)

initialise_array:
    mov [ebx], eax ; Store the value in `EAX` at address `EBX`
    inc eax ; Increment `EAX` to the next number
    add ebx, 4 ; Move `EBX` 4 bytes forward to the next integer
    dec ecx ; Decrement loop counter `ECX`
    cmp ecx, 0 ; Check if `ECX` is zero
    jne initialise_array ; Repeat until all values are stored

    ; --- Prompt for the range input ---
    
    mov eax, prompt_start ; Load prompt for start of range into EAX
    call print_string ; Display the start range prompt
    call read_int ; Read the start range input
    mov [start_range], eax ; Store the start of the range
    
    mov eax, prompt_end ; Load prompt for end of range into EAX
    call print_string ; Display the end range prompt
    call read_int ; Read the end range input
    mov [end_range], eax ; Store the end of the range

    ; --- Validate the range ---

    mov eax, [start_range] ; Load start_range into EAX
    cmp eax, 1 ; Ensure start_range is at least 1
    jl range_error ; Jump to error if less than 1
    mov ebx, [end_range] ; Load end_range into EBX
    cmp ebx, 100 ; Ensure end_range is at most 100
    jg range_error ; Jump to error if more than 100
    cmp eax, ebx ; Ensure start is less than or equal to end
    jg range_error ; Jump to error if invalid range

    ; --- Summing the array within the given range ---

    mov ecx, eax ; Load start_range as starting index into ECX
    dec ecx ; Convert to zero-based index
    mov ebx, [end_range] ; Load end_range to EBX
    sub ebx, eax ; Calculate the range length
    inc ebx ; Adjust range to include endpoint
    mov edx, 0 ; Initialize sum accumulator in EDX
    lea edi, [array] ; Load base address of array into EDI

sum_array:
    add edx, [edi + ecx*4] ; Add current element to sum
    inc ecx ; Move to next index
    dec ebx ; Decrement range length
    jnz sum_array ; Repeat until range is covered

    mov [sum], edx ; Store the calculated sum

    ; --- Display the result of sum within range ---
    
    mov eax, sum_result_msg ; Load result message into EAX
    call print_string ; Display the result message
    mov eax, [sum] ; Load calculated sum into EAX
    call print_int ; Print the sum
    call print_nl ; Print a newline

    ; --- Programme exit point ---
    
    popa ; Restore original values of registers
    ret ; Return from the main function

display_error:
    mov eax, error_msg ; Load error message address
    call print_string ; Display error message
    call print_nl ; Print newline after error message
    jmp exit_programme ; Jump to program exit

range_error:
    mov eax, range_error_msg ; Load range error message address
    call print_string ; Display range error message
    call print_nl ; Print newline after range error

exit_programme:
    popa ; Restore states of the registers
    mov eax, 0 ; Signal successful completion
    ret ; Return from the function
